@@ -89,6 +89,10 @@ void newtCheckboxSetValue(newtComponent co, char value) {
     cbDraw(co);
 }
 
+/* 
+ * returns NULL on error.
+ * FIXME: Check all calls.
+ */
 newtComponent newtCheckbox(int left, int top, const char * text, char defValue,
 			   const char * seq, char * result) {
     newtComponent co;
@@ -97,7 +101,13 @@ newtComponent newtCheckbox(int left, int top, const char * text, char defValue,
     if (!seq) seq = " *";
 
     co = malloc(sizeof(*co));
+    if (co == NULL)
+   	return NULL;
     cb = malloc(sizeof(struct checkbox));
+    if (cb == NULL) {
+	free(co);
+	return NULL;
+    }
     co->data = cb;
     cb->flags = 0;
     if (result)
@@ -117,7 +127,7 @@ newtComponent newtCheckbox(int left, int top, const char * text, char defValue,
 
     co->callback = NULL;
     co->height = 1;
-    co->width = strlen(text) + 4;
+    co->width = wstrlen(text, -1) + 4;
     co->top = top;
     co->left = left;
     co->takesFocus = 1;
@@ -131,6 +141,12 @@ void newtCheckboxSetFlags(newtComponent co, int flags, enum newtFlagsSense sense
 
     cb->flags = newtSetFlags(cb->flags, flags, sense);
 
+    // If the flag just sets a property (eg. NEWT_FLAG_RETURNEXIT),
+    // don't redraw, etc. as the component might be 'hidden' and not to
+    // be drawn (eg. in a scrolled list)
+    if (flags == NEWT_FLAG_RETURNEXIT)
+	    return;
+
     if (!(cb->flags & NEWT_FLAG_DISABLED))
 	co->takesFocus = 1;
     else
@@ -232,7 +248,10 @@ struct eventResult cbEvent(newtComponent co, struct event ev) {
 		    er.result = ER_IGNORED;
 		}
 	    } else if(ev.u.key == NEWT_KEY_ENTER) {
-		er.result = ER_IGNORED;
+		if (cb->flags & NEWT_FLAG_RETURNEXIT)
+			er.result = ER_EXITFORM;
+		else
+			er.result = ER_IGNORED;
 	    } else {
 		er.result = ER_IGNORED;
 	    }
