@@ -1,31 +1,54 @@
 /* simple dialog boxes, used by both whiptail and tcl dialog bindings */
 
+#include "config.h"
 #include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <wchar.h>
+#include <slang.h>
 
+#include "nls.h"
 #include "dialogboxes.h"
 #include "newt.h"
+#include "newt_pr.h"
 #include "popt.h"
 
+#define MAXBUF    200
+#define MAXFORMAT 20
+
 /* globals -- ick */
 static int buttonHeight = 1;
+
+int max (int a, int b)
+{
+	return (a > b) ? a : b;
+}
+
+int min (int a, int b)
+{
+	return ( a < b) ? a : b ;
+}
+ 
 static newtComponent (*makeButton)(int left, int right, const char * text) = 
 		newtCompactButton;
 
 static void addButtons(int height, int width, newtComponent form, 
 		       newtComponent * okay, newtComponent * cancel, 
 		       int flags) {
+	// FIXME: DO SOMETHING ABOUT THE HARD-CODED CONSTANTS
     if (flags & FLAG_NOCANCEL) {
-	*okay = makeButton((width - 8) / 2, height - buttonHeight - 1, "Ok");
-	*cancel = NULL;
+	   *okay = makeButton((width - 8) / 2, height - buttonHeight - 1,
+			      dgettext(PACKAGE, "Ok"));
+	    *cancel = NULL;
 	newtFormAddComponent(form, *okay);
     } else {
-	*okay = makeButton((width - 18) / 3, height - buttonHeight - 1, "Ok");
+	*okay = makeButton((width - 18) / 3, height - buttonHeight - 1, 
+			   dgettext(PACKAGE,"Ok"));
 	*cancel = makeButton(((width - 18) / 3) * 2 + 9, 
-				height - buttonHeight - 1, "Cancel");
+				height - buttonHeight - 1, 
+				dgettext(PACKAGE,"Cancel"));
 	newtFormAddComponents(form, *okay, *cancel, NULL);
     }
 }
@@ -128,9 +151,10 @@ int gauge(const char * text, int height, int width, poptContext optCon, int fd,
 }
 
 int inputBox(const char * text, int height, int width, poptContext optCon, 
-		int flags, char ** result) {
+		int flags, const char ** result) {
     newtComponent form, entry, okay, cancel, answer, tb;
-    char * val;
+    const char * val;
+    int pFlag = (flags & FLAG_PASSWORD) ? NEWT_FLAG_PASSWORD : 0;
     int rc = DLG_OKAY;
     int top;
 
@@ -140,7 +164,7 @@ int inputBox(const char * text, int height, int width, poptContext optCon,
 
     form = newtForm(NULL, NULL, 0);
     entry = newtEntry(1, top + 1, val, width - 2, &val, 
-			NEWT_FLAG_SCROLL | NEWT_FLAG_RETURNEXIT);
+			NEWT_FLAG_SCROLL | NEWT_FLAG_RETURNEXIT | pFlag);
 
     newtFormAddComponents(form, tb, entry, NULL);
 
@@ -149,14 +173,45 @@ int inputBox(const char * text, int height, int width, poptContext optCon,
     answer = newtRunForm(form);
     if (answer == cancel)
 	rc = DLG_CANCEL;
+    else if (answer == NULL)
+	rc = DLG_ESCAPE;
 
     *result = val;
 
     return rc;
 }
 
+static int mystrncpyw(char *dest, const char *src, int n, int *maxwidth)
+{
+    int i = 0;
+    int w = 0, cw;
+    wchar_t c;
+    mbstate_t ps;
+    const char *p = src;
+    char *d = dest;
+    
+    memset(&ps, 0, sizeof(ps));
+    
+    for (;;) {
+	int ret = mbrtowc(&c, p, MB_CUR_MAX, &ps);
+	if (ret <= 0) break;
+	if (ret + i >= n) break;
+	cw = wcwidth(c);
+	if (cw < 0) break;
+	if (cw + w > *maxwidth) break;
+	w += cw;
+	memcpy(d, p, ret);
+	d += ret;
+	p += ret;
+	i += ret;
+    }
+    dest[i] = '\0';
+    *maxwidth = w;
+    return i;
+}
+
 int listBox(const char * text, int height, int width, poptContext optCon,
-		int flags, char ** result) {
+		int flags, const char *default_item, const char ** result) {
     newtComponent form, okay, tb, answer, listBox;
     newtComponent cancel = NULL;
     const char * arg;
@@ -166,15 +221,18 @@ int listBox(const char * text, int height, int width, poptContext optCon,
     int allocedItems = 5;
     int i, top;
     int rc = DLG_OKAY;
-    char buf[80], format[20];
+    char buf[MAXBUF];
     int maxTagWidth = 0;
     int maxTextWidth = 0;
+    int defItem = -1;
     int scrollFlag;
+    int lineWidth, textWidth, tagWidth;
     struct {
 	const char * text;
 	const char * tag;
     } * itemInfo = malloc(allocedItems * sizeof(*itemInfo));
 
+    if (itemInfo == NULL) return DLG_ERROR;
     if (!(arg = poptGetArg(optCon))) return DLG_ERROR;
     listHeight = strtoul(arg, &end, 10);
     if (*end) return DLG_ERROR;
@@ -183,9 +241,13 @@ int listBox(const char * text, int height, int width, poptContext optCon,
 	if (allocedItems == numItems) {
 	    allocedItems += 5;
 	    itemInfo = realloc(itemInfo, sizeof(*itemInfo) * allocedItems);
+	    if (itemInfo == NULL) return DLG_ERROR;
 	}
 
 	itemInfo[numItems].tag = arg;
+	if (default_item && (strcmp(default_item, arg) == 0)) {
+	 	defItem = numItems;
+	}
 	if (!(arg = poptGetArg(optCon))) return DLG_ERROR;
 
 	if (!(flags & FLAG_NOITEM)) {
@@ -193,13 +255,19 @@ int listBox(const char * text, int height, int width, poptContext optCon,
 	} else
 	    itemInfo[numItems].text = "";
 
-	if (strlen(itemInfo[numItems].text) > (unsigned int)maxTextWidth)
-	    maxTextWidth = strlen(itemInfo[numItems].text);
-	if (strlen(itemInfo[numItems].tag) > (unsigned int)maxTagWidth)
-	    maxTagWidth = strlen(itemInfo[numItems].tag);
+	if (wstrlen(itemInfo[numItems].text,-1) > (unsigned int)maxTextWidth)
+	    maxTextWidth = wstrlen(itemInfo[numItems].text,-1);
+	if (wstrlen(itemInfo[numItems].tag,-1) > (unsigned int)maxTagWidth)
+	    maxTagWidth = wstrlen(itemInfo[numItems].tag,-1);
 
 	numItems++;
     }
+    if (numItems == 0)
+	return DLG_ERROR;
+
+    if (flags & FLAG_NOTAGS) {
+	    maxTagWidth = 0;
+    }
 
     form = newtForm(NULL, NULL, 0);
 
@@ -214,17 +282,47 @@ int listBox(const char * text, int height, int width, poptContext optCon,
 	i = 2;
     }
 
-    listBox = newtListbox(3 + ((width - 10 - maxTagWidth - maxTextWidth - i) 
-					/ 2),
-			  top + 1, listHeight, 
-			    NEWT_FLAG_RETURNEXIT | scrollFlag);
+    lineWidth = min(maxTagWidth + maxTextWidth + i, SLtt_Screen_Cols - 10);
+    listBox = newtListbox( (width - lineWidth) / 2 , top + 1, listHeight,
+                          NEWT_FLAG_RETURNEXIT | scrollFlag);
 
-    sprintf(format, "%%-%ds  %%s", maxTagWidth);
-    for (i = 0; i < numItems; i++) {
-	sprintf(buf, format, itemInfo[i].tag, itemInfo[i].text);
-	newtListboxAddEntry(listBox, buf, (void *) i);
+    textWidth = maxTextWidth;
+    tagWidth = maxTagWidth;
+    if (maxTextWidth == 0) {
+        tagWidth = lineWidth;
+    } else {
+       if (maxTextWidth + maxTagWidth + i > lineWidth)
+               tagWidth = textWidth = (lineWidth / 2) - 2;
+       else {
+               tagWidth++;
+               textWidth++;
+       }
     }
 
+    if (!(flags & FLAG_NOTAGS)) {
+       for (i = 0; i < numItems; i++) {
+	   int w = tagWidth;
+	   int len, j;
+	   len = mystrncpyw(buf, itemInfo[i].tag, MAXBUF, &w);
+	   for (j = 0; j < tagWidth - w; j++) {
+		   if (len >= MAXBUF) break;
+		   buf[len++] = ' ';
+	   }
+	   buf[len] = '\0';
+	   w = textWidth;
+	   mystrncpyw(buf + len, itemInfo[i].text, MAXBUF-len, &w);
+           newtListboxAddEntry(listBox, buf, (void *) i);
+       }
+     } else {
+        for (i = 0; i < numItems; i++) {
+           snprintf(buf, MAXBUF, "%s", itemInfo[i].text);
+           newtListboxAddEntry(listBox, buf, (void *) i);
+      }
+   }
+
+    if (defItem != -1)
+	newtListboxSetCurrent (listBox, defItem);
+
     newtFormAddComponents(form, tb, listBox, NULL);
 
     addButtons(height, width, form, &okay, &cancel, flags);
@@ -232,6 +330,8 @@ int listBox(const char * text, int height, int width, poptContext optCon,
     answer = newtRunForm(form);
     if (answer == cancel)
 	rc = DLG_CANCEL;
+    if (answer == NULL)
+	rc = DLG_ESCAPE;
 
     i = (int) newtListboxGetCurrent(listBox);
     *result = itemInfo[i].tag;
@@ -240,7 +340,7 @@ int listBox(const char * text, int height, int width, poptContext optCon,
 }
 
 int checkList(const char * text, int height, int width, poptContext optCon,
-		int useRadio, int flags, char *** selections) {
+		int useRadio, int flags, const char *** selections) {
     newtComponent form, okay, tb, subform, answer;
     newtComponent sb = NULL, cancel = NULL;
     const char * arg;
@@ -251,7 +351,7 @@ int checkList(const char * text, int height, int width, poptContext optCon,
     int i;
     int numSelected;
     int rc = DLG_OKAY;
-    char buf[80], format[20];
+    char buf[MAXBUF], format[MAXFORMAT];
     int maxWidth = 0;
     int top;
     struct {
@@ -261,6 +361,7 @@ int checkList(const char * text, int height, int width, poptContext optCon,
     } * cbInfo = malloc(allocedBoxes * sizeof(*cbInfo));
     char * cbStates = malloc(allocedBoxes * sizeof(cbStates));
 
+    if ( (cbInfo == NULL) || (cbStates == NULL)) return DLG_ERROR;
     if (!(arg = poptGetArg(optCon))) return DLG_ERROR;
     listHeight = strtoul(arg, &end, 10);
     if (*end) return DLG_ERROR;
@@ -270,6 +371,7 @@ int checkList(const char * text, int height, int width, poptContext optCon,
 	    allocedBoxes += 5;
 	    cbInfo = realloc(cbInfo, sizeof(*cbInfo) * allocedBoxes);
 	    cbStates = realloc(cbStates, sizeof(*cbStates) * allocedBoxes);
+	    if ((cbInfo == NULL) || (cbStates == NULL)) return DLG_ERROR;
 	}
 
 	cbInfo[numBoxes].tag = arg;
@@ -287,8 +389,8 @@ int checkList(const char * text, int height, int width, poptContext optCon,
 	else
 	    cbStates[numBoxes] = ' ';
 
-	if (strlen(cbInfo[numBoxes].tag) > (unsigned int)maxWidth)
-	    maxWidth = strlen(cbInfo[numBoxes].tag);
+	if (wstrlen(cbInfo[numBoxes].tag,-1) > (unsigned int)maxWidth)
+	    maxWidth = wstrlen(cbInfo[numBoxes].tag,-1);
 
 	numBoxes++;
     }
@@ -308,9 +410,9 @@ int checkList(const char * text, int height, int width, poptContext optCon,
     subform = newtForm(sb, NULL, 0);
     newtFormSetBackground(subform, NEWT_COLORSET_CHECKBOX);
 
-    sprintf(format, "%%-%ds  %%s", maxWidth);
+    snprintf(format, MAXFORMAT, "%%-%ds  %%s", maxWidth);
     for (i = 0; i < numBoxes; i++) {
-	sprintf(buf, format, cbInfo[i].tag, cbInfo[i].text);
+	snprintf(buf, MAXBUF, format, cbInfo[i].tag, cbInfo[i].text);
 
 	if (useRadio)
 	    cbInfo[i].comp = newtRadiobutton(4, top + 1 + i, buf,
@@ -320,6 +422,7 @@ int checkList(const char * text, int height, int width, poptContext optCon,
 	    cbInfo[i].comp = newtCheckbox(4, top + 1 + i, buf,
 			      cbStates[i], NULL, cbStates + i);
 
+	newtCheckboxSetFlags(cbInfo[i].comp, NEWT_FLAG_RETURNEXIT, NEWT_FLAGS_SET);
 	newtFormAddComponent(subform, cbInfo[i].comp);
     }
 
@@ -333,12 +436,15 @@ int checkList(const char * text, int height, int width, poptContext optCon,
     answer = newtRunForm(form);
     if (answer == cancel)
 	rc = DLG_CANCEL;
+    if (answer == NULL)
+	rc = DLG_ESCAPE;
 
     if (useRadio) {
 	answer = newtRadioGetCurrent(cbInfo[0].comp);
 	for (i = 0; i < numBoxes; i++) 
 	    if (cbInfo[i].comp == answer) {
 		*selections = malloc(sizeof(char *) * 2);
+	 	if (*selections == NULL) return DLG_ERROR;
 		(*selections)[0] = cbInfo[i].tag;
 		(*selections)[1] = NULL;
 		break;
@@ -350,6 +456,7 @@ int checkList(const char * text, int height, int width, poptContext optCon,
 	}
 
 	*selections = malloc(sizeof(char *) * (numSelected + 1));
+	if (*selections == NULL) return DLG_ERROR;
 
 	numSelected = 0;
 	for (i = 0; i < numBoxes; i++) {
@@ -366,6 +473,7 @@ int checkList(const char * text, int height, int width, poptContext optCon,
 int messageBox(const char * text, int height, int width, int type, int flags) {
     newtComponent form, yes, tb, answer;
     newtComponent no = NULL;
+    int rc = DLG_OKAY;
     int tFlag = (flags & FLAG_SCROLL_TEXT) ? NEWT_FLAG_SCROLL : 0;
 
     form = newtForm(NULL, NULL, 0);
@@ -380,13 +488,16 @@ int messageBox(const char * text, int height, int width, int type, int flags) {
     case MSGBOX_INFO:
 	break;
     case MSGBOX_MSG:
-	yes = makeButton((width - 8) / 2, height - 1 - buttonHeight, "Ok");
+	// FIXME Do something about the hard-coded constants
+	yes = makeButton((width - 8) / 2, height - 1 - buttonHeight, 
+			 dgettext(PACKAGE,"Ok"));
 	newtFormAddComponent(form, yes);
 	break;
     default:
-	yes = makeButton((width - 16) / 3, height - 1 - buttonHeight, "Yes");
+	yes = makeButton((width - 16) / 3, height - 1 - buttonHeight, 
+			 dgettext(PACKAGE,"Yes"));
 	no = makeButton(((width - 16) / 3) * 2 + 9, height - 1 - buttonHeight, 
-			"No");
+			dgettext(PACKAGE,"No"));
 	newtFormAddComponents(form, yes, no, NULL);
 
 	if (flags & FLAG_DEFAULT_NO)
@@ -394,7 +505,8 @@ int messageBox(const char * text, int height, int width, int type, int flags) {
     }
 
     if ( type != MSGBOX_INFO ) {
-	newtRunForm(form);
+	if (newtRunForm(form) == NULL)
+		rc = DLG_ESCAPE;
 
 	answer = newtFormGetCurrent(form);
 
@@ -408,7 +520,7 @@ int messageBox(const char * text, int height, int width, int type, int flags) {
 	
 
 
-    return DLG_OKAY;
+    return rc;
 }
 
 void useFullButtons(int state) {
