@@ -1,15 +1,18 @@
 #include "config.h"
 
+#include <sys/types.h>
+
 #include <slang.h>
 #include <stdarg.h>
 #include <stdlib.h>
+#ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
+#endif
 #include <sys/time.h>
 
 #ifdef USE_GPM
 #include <ctype.h>
 #include <sys/time.h>      /* timeval */
-#include <sys/types.h>     /* socket() */
 #include <sys/socket.h>    /* socket() */
 #include <sys/un.h>        /* struct sockaddr_un */
 #include <sys/fcntl.h>     /* O_RDONLY */
@@ -468,6 +471,7 @@ newtComponent newtForm(newtComponent vertBar, void * help, int flags) {
     if (!(form->flags & NEWT_FLAG_NOF12)) {
 	newtFormAddHotKey(co, NEWT_KEY_F12);
     }
+    /* TEST */ newtFormAddHotKey (co, NEWT_KEY_ESCAPE);
 
     if (vertBar)
 	form->vertBar = vertBar;
@@ -483,6 +487,7 @@ newtComponent newtForm(newtComponent vertBar, void * help, int flags) {
 newtComponent newtFormGetCurrent(newtComponent co) {
     struct form * form = co->data;
 
+    if (form->currComp == -1) return 0;
     return form->elements[form->currComp].co;
 }
 
@@ -590,6 +595,7 @@ void newtDrawForm(newtComponent co) {
 
     SLsmg_set_color(form->background);
     newtClearBox(co->left, co->top, co->width, co->height);
+
     for (i = 0, el = form->elements; i < form->numComps; i++, el++) {
 	/* the scrollbar *always* fits somewhere */
 	if (el->co == form->vertBar) {
@@ -624,7 +630,7 @@ static struct eventResult formEvent(newtComponent co, struct event ev) {
     er.result = ER_IGNORED;
     if (!form->numComps) return er;
 
-    subco = form->elements[form->currComp].co;
+    if (form->currComp == -1) return er;
 
     switch (ev.when) {
       case EV_EARLY:
@@ -884,7 +890,7 @@ void newtFormRun(newtComponent co, struct newtExitStruct * es) {
     struct eventResult er;
     int key, i, max;
     int done = 0;
-    fd_set readSet, writeSet;
+    fd_set readSet, writeSet, exceptSet;
     struct timeval nextTimeout, now, timeout;
 #ifdef USE_GPM
     int x, y;
@@ -914,6 +920,7 @@ void newtFormRun(newtComponent co, struct newtExitStruct * es) {
 
 	FD_ZERO(&readSet);
 	FD_ZERO(&writeSet);
+	FD_ZERO(&exceptSet);
 	FD_SET(0, &readSet);
 #ifdef USE_GPM
 	if (gpm_fd > 0) {
@@ -929,6 +936,8 @@ void newtFormRun(newtComponent co, struct newtExitStruct * es) {
 		FD_SET(form->fds[i].fd, &readSet);
 	    if (form->fds[i].flags & NEWT_FD_WRITE)
 		FD_SET(form->fds[i].fd, &writeSet);
+	    if (form->fds[i].flags & NEWT_FD_EXCEPT)
+		FD_SET(form->fds[i].fd, &exceptSet);
 	}
 
 	if (form->timer) {
@@ -965,7 +974,7 @@ void newtFormRun(newtComponent co, struct newtExitStruct * es) {
 	    timeout.tv_sec = timeout.tv_usec = 0;
 	}
 
-	i = select(max + 1, &readSet, &writeSet, NULL, 
+	i = select(max + 1, &readSet, &writeSet, &exceptSet, 
 			form->timer ? &timeout : NULL);
 	if (i < 0) continue;	/* ?? What should we do here? */
 
@@ -1005,7 +1014,7 @@ void newtFormRun(newtComponent co, struct newtExitStruct * es) {
 		key = newtGetKey();
 
 		if (key == NEWT_KEY_RESIZE) {
-		    /* newtResizeScreen(1); */
+		    newtResizeScreen(1);
 		    continue;
 		}
 
@@ -1034,6 +1043,19 @@ void newtFormRun(newtComponent co, struct newtExitStruct * es) {
 		    }
 		}
 	    } else {
+		for (i = 0; i < form->numFds; i++) {
+		    if (((form->fds[i].flags & NEWT_FD_READ)
+			&& FD_ISSET(form->fds[i].fd, &readSet))
+			|| ((form->fds[i].flags & NEWT_FD_WRITE)
+			&& FD_ISSET(form->fds[i].fd, &writeSet))
+			|| ((form->fds[i].flags & NEWT_FD_EXCEPT)
+			&& FD_ISSET(form->fds[i].fd, &exceptSet))) break;
+		}
+		if(i < form->numFds)
+		    es->u.watch = form->fds[i].fd;
+		else
+		    es->u.watch = -1;
+
 		es->reason = NEWT_EXIT_FDREADY;
 		done = 1;
 	    }
@@ -1098,10 +1120,17 @@ void newtFormSetBackground(newtComponent co, int color) {
 
 void newtFormWatchFd(newtComponent co, int fd, int fdFlags) {
     struct form * form = co->data;
+    int i;
 
-    form->fds = realloc(form->fds, (form->numFds + 1) * sizeof(*form->fds));
-    form->fds[form->numFds].fd = fd;
-    form->fds[form->numFds++].flags = fdFlags;
+    for (i = 0; i < form->numFds; i++)
+      if (form->fds[i].fd == fd)
+	break;
+
+    if(i >= form->numFds)
+      form->fds = realloc(form->fds, (++form->numFds) * sizeof(*form->fds));
+
+    form->fds[i].fd = fd;
+    form->fds[i].flags = fdFlags;
     if (form->maxFd < fd) form->maxFd = fd;
 }
 
