@@ -32,7 +32,7 @@ newtComponent newtLabel(int left, int top, const char * text) {
     co->ops = &labelOps;
 
     co->height = 1;
-    co->width = strlen(text);
+    co->width = wstrlen(text, -1);
     co->top = top;
     co->left = left;
     co->takesFocus = 0;
@@ -47,6 +47,7 @@ void newtLabelSetText(newtComponent co, const char * text) {
     int newLength;
     struct label * la = co->data;
 
+    co->width = wstrlen(text,-1);
     newLength = strlen(text);
     if (newLength <= la->length) {
 	memset(la->text, ' ', la->length);
@@ -55,7 +56,6 @@ void newtLabelSetText(newtComponent co, const char * text) {
 	free(la->text);
 	la->text = strdup(text);
 	la->length = newLength;
-	co->width = newLength;
     }
 
     labelDraw(co);
@@ -64,7 +64,7 @@ void newtLabelSetText(newtComponent co, const char * text) {
 static void labelDraw(newtComponent co) {
     struct label * la = co->data;
 
-    if (co->top == -1) return;
+    if (co->isMapped == -1) return;
 
     SLsmg_set_color(COLORSET_LABEL);
 
