@@ -73,8 +73,13 @@ enum newtFlagsSense { NEWT_FLAGS_SET, NEWT_FLAGS_RESET, NEWT_FLAGS_TOGGLE };
 #define NEWT_FLAG_SELECTED	(1 << 9)
 #define NEWT_FLAG_CHECKBOX	(1 << 10)
 #define NEWT_FLAG_PASSWORD      (1 << 11)  /* draw '*'  of chars in entrybox */
+#define NEWT_FLAG_SHOWCURSOR    (1 << 12) /* Only applies to listbox for now */
 #define NEWT_FD_READ		(1 << 0)
 #define NEWT_FD_WRITE		(1 << 1)
+#define NEWT_FD_EXCEPT		(1 << 2)
+
+#define NEWT_CHECKBOXTREE_UNSELECTABLE	(1 << 12)
+#define NEWT_CHECKBOXTREE_HIDE_BOX	(1 << 13)
 
 #define NEWT_CHECKBOXTREE_COLLAPSED	'\0'
 #define NEWT_CHECKBOXTREE_EXPANDED	'\1'
@@ -108,18 +113,19 @@ void newtCls(void);
 void newtResizeScreen(int redraw);
 void newtWaitForKey(void);
 void newtClearKeyBuffer(void);
-void newtDelay(int usecs);
+void newtDelay(unsigned int usecs);
 /* top, left are *not* counting the border */
-int newtOpenWindow(int left, int top, int width, int height, 
-			  const char * title);
-int newtCenteredWindow(int width, int height, const char * title);
+int newtOpenWindow(unsigned int left,unsigned int top, 
+		   unsigned int width,unsigned  int height, 
+ 		  const char * title);
+int newtCenteredWindow(unsigned int width,unsigned int height, const char * title);
 void newtPopWindow(void);
 void newtSetColors(struct newtColors colors);
 void newtRefresh(void);
 void newtSuspend(void);
 void newtSetSuspendCallback(newtSuspendCallback cb, void * data);
 void newtSetHelpCallback(newtCallback cb);
-void newtResume(void);
+int  newtResume(void);
 void newtPushHelpLine(const char * text);
 void newtRedrawHelpLine(void);
 void newtPopHelpLine(void);
@@ -173,11 +179,14 @@ void **newtListboxGetSelection(newtComponent co, int *numitems);
 void newtListboxClearSelection(newtComponent co);
 void newtListboxSelectItem(newtComponent co, const void * key,
 	enum newtFlagsSense sense);
+/* Returns number of items currently in listbox. */
+int newtListboxItemCount(newtComponent co);
 
 newtComponent newtCheckboxTree(int left, int top, int height, int flags);
 newtComponent newtCheckboxTreeMulti(int left, int top, int height, char *seq, int flags);
 const void ** newtCheckboxTreeGetSelection(newtComponent co, int *numitems);
 const void * newtCheckboxTreeGetCurrent(newtComponent co);
+void newtCheckboxTreeSetCurrent(newtComponent co, void * item);
 const void ** newtCheckboxTreeGetMultiSelection(newtComponent co, int *numitems, char seqnum);
 /* last item is NEWT_ARG_LAST for all of these */
 int newtCheckboxTreeAddItem(newtComponent co, 
@@ -189,10 +198,11 @@ int newtCheckboxTreeAddArray(newtComponent co,
 int * newtCheckboxTreeFindItem(newtComponent co, void * data);
 void newtCheckboxTreeSetEntry(newtComponent co, const void * data,
 			      const char * text);
+void newtCheckboxTreeSetWidth(newtComponent co, int width);
 char newtCheckboxTreeGetEntryValue(newtComponent co, const void * data);
 void newtCheckboxTreeSetEntryValue(newtComponent co, const void * data,
 				   char value);
-    
+ 
 newtComponent newtTextboxReflowed(int left, int top, char * text, int width,
 				  int flexDown, int flexUp, int flags);
 newtComponent newtTextbox(int left, int top, int width, int height, int flags);
@@ -206,6 +216,7 @@ struct newtExitStruct {
     enum { NEWT_EXIT_HOTKEY, NEWT_EXIT_COMPONENT, NEWT_EXIT_FDREADY,
 	   NEWT_EXIT_TIMER } reason;
     union {
+	int watch;
 	int key;
 	newtComponent co;
     } u;
@@ -230,7 +241,7 @@ void newtFormAddHotKey(newtComponent co, int key);
 typedef int (*newtEntryFilter)(newtComponent entry, void * data, int ch,
 			       int cursor);
 newtComponent newtEntry(int left, int top, const char * initialValue, int width,
-			char ** resultPtr, int flags);
+			const char ** resultPtr, int flags);
 void newtEntrySet(newtComponent co, const char * value, int cursorAtEnd);
 void newtEntrySetFilter(newtComponent co, newtEntryFilter filter, void * data);
 char * newtEntryGetValue(newtComponent co);
@@ -251,6 +262,7 @@ void newtFormDestroy(newtComponent form);
 #define NEWT_KEY_TAB			'\t'
 #define NEWT_KEY_ENTER			'\r'
 #define NEWT_KEY_SUSPEND		'\032'			/* ctrl - z*/
+#define NEWT_KEY_ESCAPE			''
 #define NEWT_KEY_RETURN			NEWT_KEY_ENTER
 
 #define NEWT_KEY_EXTRA_BASE		0x8000
@@ -345,7 +357,7 @@ int newtWinMenu(char * title, char * text, int suggestedWidth, int flexDown,
 
 struct newtWinEntry {
     char * text;
-    char ** value;		/* may be initialized to set default */
+    const char ** value;		/* may be initialized to set default */
     int flags;
 };
 
