@@ -26,6 +26,7 @@ void newtDrawBox(int left, int top, int width, int height, int shadow);
 void newtClearBox(int left, int top, int width, int height);
 
 int newtGetKey(void);
+void newtTrashScreen(void);
 
 struct newtComponent_struct {
     /* common data */
@@ -79,4 +80,7 @@ void newtDefaultMappedHandler(newtComponent c, int isMapped);
 struct eventResult newtDefaultEventHandler(newtComponent c,
 					   struct event ev);
 
+int _newt_wstrlen(const char *str, int len);
+#define wstrlen(str,len) _newt_wstrlen((str),(len))
+
 #endif /* H_NEWT_PR */
