@@ -1,4 +1,4 @@
-#include <slang/slang.h>
+#include <slang.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -8,6 +8,7 @@
 struct scrollbar {
     int curr;
     int cs, csThumb;
+    int arrows;
 } ;
 
 static void sbDraw(newtComponent co);
@@ -18,14 +19,18 @@ static struct componentOps sbOps = {
     sbDraw,
     newtDefaultEventHandler,
     sbDestroy,
-    NULL,
+    newtDefaultPlaceHandler,
+    newtDefaultMappedHandler,
 } ;
 
 void newtScrollbarSet(newtComponent co, int where, int total) {
     struct scrollbar * sb = co->data;
     int new;
 
-    new = (where * (co->height - 1)) / (total ? total : 1);
+    if (sb->arrows)
+	new = (where * (co->height - 3)) / (total ? total : 1) + 1;
+    else
+	new = (where * (co->height - 1)) / (total ? total : 1);
     if (new != sb->curr) {
 	sbDrawThumb(co, 0);
 	sb->curr = new;
@@ -42,11 +47,18 @@ newtComponent newtVerticalScrollbar(int left, int top, int height,
     sb = malloc(sizeof(*sb));
     co->data = sb;
 
-    sb->curr = 0;
+    if (!strcmp(getenv("TERM"), "linux") && height >= 2) {
+	sb->arrows = 1;
+	sb->curr = 1;
+    } else {
+	sb->arrows = 0;
+	sb->curr = 0;
+    }
     sb->cs = normalColorset;
     sb->csThumb = thumbColorset;
 
     co->ops = &sbOps;
+    co->isMapped = 0;
     co->left = left;
     co->top = top;
     co->height = height;
@@ -60,12 +72,25 @@ static void sbDraw(newtComponent co) {
     struct scrollbar * sb = co->data;
     int i;
 
+    if (!co->isMapped) return;
+
     SLsmg_set_color(sb->cs);
 
     SLsmg_set_char_set(1);
-    for (i = 0; i < co->height; i++) {
-	newtGotorc(i + co->top, co->left);
-	SLsmg_write_char('\x61');
+    if (sb->arrows) {
+	newtGotorc(co->top, co->left);
+	SLsmg_write_char(SLSMG_UARROW_CHAR);
+	for (i = 1; i < co->height - 1; i++) {
+	    newtGotorc(i + co->top, co->left);
+	    SLsmg_write_char(SLSMG_CKBRD_CHAR);
+	}
+	newtGotorc(co->top + co->height - 1, co->left);
+	SLsmg_write_char(SLSMG_DARROW_CHAR);
+    } else {
+	for (i = 0; i < co->height; i++) {
+	    newtGotorc(i + co->top, co->left);
+	    SLsmg_write_char(SLSMG_CKBRD_CHAR);
+	}
     }
 
     SLsmg_set_char_set(0);
@@ -75,7 +100,9 @@ static void sbDraw(newtComponent co) {
 
 static void sbDrawThumb(newtComponent co, int isOn) {
     struct scrollbar * sb = co->data;
-    char ch = isOn ? '#' : '\x61';
+    SLtt_Char_Type ch = isOn ? '#' : SLSMG_CKBRD_CHAR;
+
+    if (!co->isMapped) return;
 
     newtGotorc(sb->curr + co->top, co->left);
     SLsmg_set_char_set(1);
