@@ -1,3 +1,10 @@
+
+#include "config.h"
+
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
+
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -5,7 +12,9 @@
 #include <unistd.h>
 
 #include "Python.h"
+#include "nls.h"
 #include "newt.h"
+#include "newt_pr.h"
 
 typedef struct snackWidget_s snackWidget;
 typedef struct snackGrid_s snackGrid;
@@ -15,12 +24,16 @@ struct callbackStruct {
     PyObject * cb, * data;
 };
 
+/* Integer to pointer, 64-bit-sane */
+#define I2P(x) ((void *)(long)(x))
+
 static struct callbackStruct suspend;
 static struct callbackStruct helpCallback;
 
 static void emptyDestructor(PyObject * s);
 
 static snackWidget * buttonWidget(PyObject * s, PyObject * args);
+static snackWidget * compactbuttonWidget(PyObject * s, PyObject * args);
 static PyObject * centeredWindow(PyObject * s, PyObject * args);
 static snackWidget * checkboxWidget(PyObject * s, PyObject * args);
 static PyObject * choiceWindow(PyObject * s, PyObject * args);
@@ -51,10 +64,12 @@ static PyObject * setHelpCallback(PyObject * s, PyObject * args);
 static PyObject * reflowText(PyObject * s, PyObject * args);
 static snackWidget * textWidget(PyObject * s, PyObject * args);
 static PyObject * ternaryWindow(PyObject * s, PyObject * args);
-static snackWidget * checkboxTreeWidget(PyObject * s, PyObject * args);
+static snackWidget * checkboxTreeWidget(PyObject * s, PyObject * args, PyObject * kwargs);
+static PyObject * pywstrlen(PyObject * s, PyObject * args);
 
 static PyMethodDef snackModuleMethods[] = {
     { "button", (PyCFunction) buttonWidget, METH_VARARGS, NULL },
+    { "compactbutton", (PyCFunction) compactbuttonWidget, METH_VARARGS, NULL },
     { "checkbox", (PyCFunction) checkboxWidget, METH_VARARGS, NULL },
     { "choice", choiceWindow, METH_VARARGS, NULL },
     { "centeredwindow", centeredWindow, METH_VARARGS, NULL },
@@ -83,12 +98,13 @@ static PyMethodDef snackModuleMethods[] = {
     { "suspendcallback", setSuspendCallback, METH_VARARGS, NULL },
     { "ternary", ternaryWindow, METH_VARARGS, NULL },
     { "textbox", (PyCFunction) textWidget, METH_VARARGS, NULL },
-    { "checkboxtree", (PyCFunction) checkboxTreeWidget, METH_VARARGS, NULL },
+    { "checkboxtree", (PyCFunction) checkboxTreeWidget, METH_VARARGS | METH_KEYWORDS, NULL },
+    { "wstrlen", (PyCFunction) pywstrlen, METH_VARARGS | METH_KEYWORDS, NULL },
     { NULL }
 } ;
 
 struct snackGrid_s {
-    PyObject_HEAD;
+    PyObject_HEAD
     newtGrid grid;
 } ;
 
@@ -120,7 +136,7 @@ static PyTypeObject snackGridType = {
 };
 
 struct snackForm_s {
-    PyObject_HEAD;
+    PyObject_HEAD
     newtComponent fo;
 } ;
 
@@ -131,6 +147,7 @@ static PyObject * formRun(snackForm * s, PyObject * args);
 static PyObject * formHotKey(snackForm * s, PyObject * args);
 static PyObject * formSetCurrent(snackForm * form, PyObject * args);
 static PyObject * formSetTimer(snackForm * form, PyObject * args);
+static PyObject * formWatchFD(snackForm * form, PyObject * args);
 
 static PyMethodDef formMethods[] = {
     { "add", (PyCFunction) formAdd, METH_VARARGS, NULL },
@@ -139,6 +156,7 @@ static PyMethodDef formMethods[] = {
     { "addhotkey", (PyCFunction) formHotKey, METH_VARARGS, NULL },
     { "setcurrent", (PyCFunction) formSetCurrent, METH_VARARGS, NULL },
     { "settimer", (PyCFunction) formSetTimer, METH_VARARGS, NULL },
+    { "watchfd", (PyCFunction) formWatchFD, METH_VARARGS, NULL },
     { NULL }
 };
 
@@ -160,7 +178,7 @@ static PyTypeObject snackFormType = {
 };
 
 struct snackWidget_s {
-    PyObject_HEAD;
+    PyObject_HEAD
     newtComponent co;
     char achar;
     void * apointer;
@@ -179,11 +197,14 @@ static PyObject * widgetListboxIns(snackWidget * s, PyObject * args);
 static PyObject * widgetListboxDel(snackWidget * s, PyObject * args);
 static PyObject * widgetListboxGet(snackWidget * s, PyObject * args);
 static PyObject * widgetListboxSet(snackWidget * s, PyObject * args);
+static PyObject * widgetListboxClear(snackWidget * s, PyObject * args);
 static PyObject * widgetTextboxText(snackWidget * s, PyObject * args);
 static PyObject * widgetCheckboxTreeAddItem(snackWidget * s, PyObject * args);
 static PyObject * widgetCheckboxTreeGetSel(snackWidget * s, PyObject * args);
 static PyObject * widgetCheckboxTreeGetCur(snackWidget * s, PyObject * args);
 static PyObject * widgetCheckboxTreeSetEntry(snackWidget * s, PyObject * args);
+static PyObject * widgetCheckboxTreeSetWidth(snackWidget * s, PyObject * args);
+static PyObject * widgetCheckboxTreeSetCurrent(snackWidget * s, PyObject * args);
 static PyObject * widgetCheckboxTreeSetEntryValue(snackWidget * s, PyObject * args);
 static PyObject * widgetCheckboxTreeGetEntryValue(snackWidget * s, PyObject * args);
 static PyObject * widgetEntrySetFlags(snackWidget * s, PyObject * args);
@@ -201,6 +222,7 @@ static PyMethodDef widgetMethods[] = {
     { "listboxSetCurrent", (PyCFunction) widgetListboxSet, METH_VARARGS, NULL },
     { "listboxSetWidth", (PyCFunction) widgetListboxSetW, METH_VARARGS, NULL },
     { "listboxDeleteItem", (PyCFunction) widgetListboxDel, METH_VARARGS, NULL },
+    { "listboxClear", (PyCFunction) widgetListboxClear, METH_VARARGS, NULL },
     { "scaleSet", (PyCFunction) scaleSet, METH_VARARGS, NULL },
     { "checkboxtreeAddItem", (PyCFunction) widgetCheckboxTreeAddItem,
       METH_VARARGS, NULL },
@@ -210,10 +232,13 @@ static PyMethodDef widgetMethods[] = {
       METH_VARARGS, NULL },
     { "checkboxtreeSetEntry", (PyCFunction) widgetCheckboxTreeSetEntry,
       METH_VARARGS, NULL },
+    { "checkboxtreeSetWidth", (PyCFunction) widgetCheckboxTreeSetWidth, METH_VARARGS, NULL },
+    { "checkboxtreeSetCurrent", (PyCFunction) widgetCheckboxTreeSetCurrent,
+      METH_VARARGS, NULL },
     { "checkboxtreeSetEntryValue", (PyCFunction) widgetCheckboxTreeSetEntryValue,
       METH_VARARGS, NULL },
     { "checkboxtreeGetSelection", (PyCFunction) widgetCheckboxTreeGetSel,
-      METH_VARARGS, NULL },
+      METH_VARARGS, NULL },  
     { "entrySetFlags", (PyCFunction) widgetEntrySetFlags, METH_VARARGS, NULL },
     { "checkboxSetFlags", (PyCFunction) widgetCheckboxSetFlags, METH_VARARGS, NULL },
     { "checkboxSetValue", (PyCFunction) widgetCheckboxSetValue, METH_VARARGS, NULL },
@@ -558,6 +583,18 @@ static snackWidget * buttonWidget(PyObject * s, PyObject * args) {
     return widget;
 }
 
+static snackWidget * compactbuttonWidget(PyObject * s, PyObject * args) {
+    snackWidget * widget;
+    char * label;
+
+    if (!PyArg_ParseTuple(args, "s", &label)) return NULL;
+
+    widget = snackWidgetNew ();
+    widget->co = newtCompactButton(-1, -1, label);
+
+    return widget;
+}
+
 static snackWidget * labelWidget(PyObject * s, PyObject * args) {
     char * label;
     snackWidget * widget;
@@ -595,15 +632,17 @@ static PyObject * widgetTextboxText(snackWidget * s, PyObject * args) {
 static snackWidget * listboxWidget(PyObject * s, PyObject * args) {
     snackWidget * widget;
     int height;
-    int doScroll = 0, returnExit = 0 ;
+    int doScroll = 0, returnExit = 0, showCursor = 0 ;
 
-    if (!PyArg_ParseTuple(args, "i|ii", &height, &doScroll, &returnExit))
+    if (!PyArg_ParseTuple(args, "i|iii", &height, &doScroll, &returnExit, &showCursor))
 	return NULL;
 
     widget = snackWidgetNew ();
     widget->co = newtListbox(-1, -1, height,
-				(doScroll ? NEWT_FLAG_SCROLL : 0) |
-				(returnExit ? NEWT_FLAG_RETURNEXIT : 0));
+			     (doScroll ? NEWT_FLAG_SCROLL : 0) |
+			     (returnExit ? NEWT_FLAG_RETURNEXIT : 0) |
+			     (showCursor ? NEWT_FLAG_SHOWCURSOR : 0)
+			     );
     widget->anint = 1;
     
     return widget;
@@ -692,7 +731,8 @@ static snackWidget * entryWidget(PyObject * s, PyObject * args) {
 			  &isHidden, &isPassword, &isScrolled, &returnExit)) return NULL;
 
     widget = snackWidgetNew ();
-    widget->co = newtEntry(-1, -1, initial, width, (char **) &widget->apointer, 
+    widget->co = newtEntry(-1, -1, initial, width,
+                           (const char **) &widget->apointer, 
 			   (isHidden ? NEWT_FLAG_HIDDEN : 0) |
 			   (isPassword ? NEWT_FLAG_PASSWORD : 0) |
 			   (returnExit ? NEWT_FLAG_RETURNEXIT : 0) |
@@ -809,6 +849,8 @@ static PyObject * formRun(snackForm * s, PyObject * args) {
 	return Py_BuildValue("(si)", "hotkey", result.u.key);
     else if (result.reason == NEWT_EXIT_TIMER)
 	return Py_BuildValue("(si)", "timer", 0);
+    else if (result.reason == NEWT_EXIT_FDREADY)
+	return Py_BuildValue("(si)", "fdready", result.u.watch);
     else
 	return Py_BuildValue("(si)", "widget", result.u.co);
 }
@@ -837,6 +879,18 @@ static PyObject * formSetTimer(snackForm * form, PyObject * args) {
     return Py_None;
 }
 
+static PyObject * formWatchFD(snackForm * form, PyObject * args) {
+    int fd, fdflags;
+
+    if (!PyArg_ParseTuple(args, "ii", &fd, &fdflags))
+	return NULL;
+
+    newtFormWatchFd(form->fo, fd, fdflags);
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject * formSetCurrent(snackForm * form, PyObject * args) {
     snackWidget * w;
 
@@ -920,7 +974,7 @@ static PyObject * widgetListboxAdd(snackWidget * s, PyObject * args) {
     if (!PyArg_ParseTuple(args, "s", &text))
 	return NULL;
 
-    newtListboxAddEntry(s->co, text, (void *) s->anint);
+    newtListboxAddEntry(s->co, text, I2P(s->anint));
 
     return PyInt_FromLong(s->anint++);
 }
@@ -932,7 +986,7 @@ static PyObject * widgetListboxIns(snackWidget * s, PyObject * args) {
     if (!PyArg_ParseTuple(args, "si", &text, &key))
 	return NULL;
 
-    newtListboxInsertEntry(s->co, text, (void *) s->anint, (void *) key);
+    newtListboxInsertEntry(s->co, text, I2P(s->anint), I2P(key));
 
     return PyInt_FromLong(s->anint++);
 }
@@ -943,7 +997,7 @@ static PyObject * widgetListboxDel(snackWidget * s, PyObject * args) {
     if (!PyArg_ParseTuple(args, "i", &key))
 	return NULL;
 
-    newtListboxDeleteEntry(s->co, (void *) key);
+    newtListboxDeleteEntry(s->co, I2P(key));
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -962,7 +1016,7 @@ static PyObject * widgetListboxSet(snackWidget * s, PyObject * args) {
     if (!PyArg_ParseTuple(args, "i", &index))
 	return NULL;
 
-    newtListboxSetCurrentByKey(s->co, (void *) index);
+    newtListboxSetCurrentByKey(s->co, I2P(index));
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -980,20 +1034,38 @@ static PyObject * widgetListboxSetW(snackWidget * s, PyObject * args) {
     return Py_None;
 }
 
+static PyObject * widgetListboxClear(snackWidget * s, PyObject * args) {
+  if (!PyArg_ParseTuple(args, ""))
+    return NULL;
+
+  newtListboxClear(s->co);
+
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
 static void emptyDestructor(PyObject * s) {
 }
 
-static snackWidget * checkboxTreeWidget(PyObject * s, PyObject * args) {
+static snackWidget * checkboxTreeWidget(PyObject * s, PyObject * args, PyObject * kwargs) {
     int height;
     int scrollBar = 0;
+    int hide_checkbox = 0;
+    int unselectable = 0;
+    int flags;
     snackWidget * widget;
+    const char *kw[] = {"height", "scrollbar", "hide_checkbox", "unselectable", NULL};
     
-    if (!PyArg_ParseTuple(args, "i|i", &height, &scrollBar))
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "i|iii", (char **) kw,
+		&height, &scrollBar, &hide_checkbox, &unselectable))
 	return NULL;
 
+    flags = (scrollBar ? NEWT_FLAG_SCROLL : 0) |
+	(hide_checkbox ? NEWT_CHECKBOXTREE_HIDE_BOX : 0) |    
+	(unselectable ? NEWT_CHECKBOXTREE_UNSELECTABLE : 0);
+
     widget = snackWidgetNew ();
-    widget->co = newtCheckboxTree(-1, -1, height,
-				  scrollBar ? NEWT_FLAG_SCROLL : 0);
+    widget->co = newtCheckboxTree(-1, -1, height, flags);
 
     widget->anint = 1;
 
@@ -1019,7 +1091,7 @@ static PyObject * widgetCheckboxTreeAddItem(snackWidget * s, PyObject * args) {
     }
     path[len] = NEWT_ARG_LAST;
 
-    newtCheckboxTreeAddArray(s->co, text, (void *) s->anint,
+    newtCheckboxTreeAddArray(s->co, text, I2P(s->anint),
     			     selected ? NEWT_FLAG_SELECTED : 0, path);
 
     return PyInt_FromLong(s->anint++);
@@ -1029,7 +1101,7 @@ static PyObject * widgetCheckboxTreeGetCur(snackWidget * s, PyObject * args) {
     if (!PyArg_ParseTuple(args, ""))
 	return NULL;
 
-    return PyInt_FromLong((int) newtCheckboxTreeGetCurrent(s->co));
+    return PyInt_FromLong((long)newtCheckboxTreeGetCurrent(s->co));
 }
 
 static PyObject * widgetCheckboxTreeSetEntry(snackWidget * s, PyObject * args) {
@@ -1038,7 +1110,30 @@ static PyObject * widgetCheckboxTreeSetEntry(snackWidget * s, PyObject * args) {
 
     if (!PyArg_ParseTuple(args, "is", &data, &text)) return NULL;
 
-    newtCheckboxTreeSetEntry(s->co, (void *)data, text);
+    newtCheckboxTreeSetEntry(s->co, I2P(data), text);
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject * widgetCheckboxTreeSetWidth(snackWidget * s, PyObject * args) {
+    int width;
+
+    if (!PyArg_ParseTuple(args, "i", &width))
+	return NULL;
+
+    newtCheckboxTreeSetWidth(s->co, width);
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject * widgetCheckboxTreeSetCurrent(snackWidget * s, PyObject * args) {
+    int data;
+
+    if (!PyArg_ParseTuple(args, "i", &data)) return NULL;
+
+    newtCheckboxTreeSetCurrent(s->co, I2P(data));
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -1050,7 +1145,7 @@ static PyObject * widgetCheckboxTreeSetEntryValue(snackWidget * s, PyObject * ar
 
     if (!PyArg_ParseTuple(args, "i|i", &data, &isOn)) return NULL;
 
-    newtCheckboxTreeSetEntryValue(s->co, (void *)data,
+    newtCheckboxTreeSetEntryValue(s->co, I2P(data),
 				  isOn ? NEWT_CHECKBOXTREE_SELECTED :
 					 NEWT_CHECKBOXTREE_UNSELECTED);
 
@@ -1066,7 +1161,7 @@ static PyObject * widgetCheckboxTreeGetEntryValue(snackWidget * s, PyObject * ar
 
     if (!PyArg_ParseTuple(args, "i", &data)) return NULL;
 
-    selection = newtCheckboxTreeGetEntryValue(s->co, (void *)data);
+    selection = newtCheckboxTreeGetEntryValue(s->co, I2P(data));
 
     if (selection == -1) return NULL;
 
@@ -1105,16 +1200,30 @@ static PyObject * widgetCheckboxTreeGetSel(snackWidget * s,
 
     sel = PyList_New(0);
     for (i = 0; i < numselected; i++) {
-    	PyList_Append(sel, PyInt_FromLong((int) selection[i]));
+    	PyList_Append(sel, PyInt_FromLong((long) selection[i]));
     }
     free(selection);
 
     return sel;
 }
 
+static PyObject * pywstrlen(PyObject * s, PyObject * args)
+{
+    char *str;
+    int len = -1;
+    
+    if (!PyArg_ParseTuple(args, "s|i", &str, &len)) return NULL;
+
+    return PyInt_FromLong(wstrlen(str, len));
+}
+
 void init_snack(void) {
     PyObject * d, * m;
 
+    setlocale (LC_ALL, "");
+    bindtextdomain (PACKAGE, LOCALEDIR);
+    textdomain (PACKAGE);
+
     m = Py_InitModule("_snack", snackModuleMethods);
     d = PyModule_GetDict(m);
 
@@ -1126,9 +1235,14 @@ void init_snack(void) {
     PyDict_SetItemString(d, "GRID_GROWX", PyInt_FromLong(NEWT_GRID_FLAG_GROWX));
     PyDict_SetItemString(d, "GRID_GROWY", PyInt_FromLong(NEWT_GRID_FLAG_GROWY));
 
+    PyDict_SetItemString(d, "FD_READ", PyInt_FromLong(NEWT_FD_READ));
+    PyDict_SetItemString(d, "FD_WRITE", PyInt_FromLong(NEWT_FD_WRITE));
+    PyDict_SetItemString(d, "FD_EXCEPT", PyInt_FromLong(NEWT_FD_EXCEPT));
+
     PyDict_SetItemString(d, "FORM_EXIT_HOTKEY", PyString_FromString("hotkey"));
     PyDict_SetItemString(d, "FORM_EXIT_WIDGET", PyString_FromString("widget"));
     PyDict_SetItemString(d, "FORM_EXIT_TIMER", PyString_FromString("timer"));
+    PyDict_SetItemString(d, "FORM_EXIT_FDREADY", PyString_FromString("fdready"));
 
     PyDict_SetItemString(d, "KEY_F1", PyInt_FromLong(NEWT_KEY_F1));
     PyDict_SetItemString(d, "KEY_F2", PyInt_FromLong(NEWT_KEY_F2));
@@ -1142,6 +1256,7 @@ void init_snack(void) {
     PyDict_SetItemString(d, "KEY_F10", PyInt_FromLong(NEWT_KEY_F10));
     PyDict_SetItemString(d, "KEY_F11", PyInt_FromLong(NEWT_KEY_F11));
     PyDict_SetItemString(d, "KEY_F12", PyInt_FromLong(NEWT_KEY_F12));
+    PyDict_SetItemString(d, "KEY_ESC", PyInt_FromLong(NEWT_KEY_ESCAPE));
 
     PyDict_SetItemString(d, "FLAG_DISABLED", PyInt_FromLong(NEWT_FLAG_DISABLED));
     PyDict_SetItemString(d, "FLAGS_SET", PyInt_FromLong(NEWT_FLAGS_SET));
